/*
 * mars3d Copyright Jan Panteltje 2005 and later.
 * 
 * This file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public  
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public   
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "tinyptc.h"

#include "xmars3d.h"

char *strsave(char *s) /* save string s somewhere */
{
char *p;

p = malloc(strlen(s) + 1);
if(!p) return 0;
strcpy(p, s);
return p;
} /* end function strsave */


void ptc_cleanup_callback(void)
{
fprintf (stderr, "Callback!\n");
} /* end function ptc_cleanup_callback */


int print_usage()
{
fprintf(stderr, "\
Usage:               mars3d [-e enlarge] [-g gamma] [-h] -l [stereo]1.pgm [-r stereo2.pgm] [-h] [-v] [-z zoom]\n\n\
-e enlarge_factor    int, >= 1, (default 1).\n\
-g gamma             float, display gamma. (default 1.0).\n\
-h                   help (this help).\n\
-l filename.pgm      first .pgm file (left one if stereo).\n\
-r filename.pgm      second stereo.pgm file (right one if stereo), if omitted display will be mono.\n\
-v                   debug (prints functions and arguments).\n\
-z zoom_factor       int, >= 1, (default 1).\n\
");

return 1;
} /* end function print_usage */


//#define DEFINED_SPEED


int main(int argc, char **argv)
{
int a, b, d, g, e, i, j, k, r;
int frame_count;
FILE *fptr1 = 0; // -Wall
FILE *fptr2 = 0; // -Wall
char *pgm_buffer_left = 0;
char *pgm_buffer_right = 0;
char *ptri;
int x_size, y_size, x_size2, y_size2;
static int *ptro;
int xpos2, ypos2;
#ifdef DEFINED_SPEED
struct timeval *start_timeval, *current_timeval;
unsigned long lstart_usecs, lcurrent_usecs, ldiff_usecs;
#endif
int view_width, view_height; // area om mars in pixels
int header_size_left, header_size_right, header_size_red, header_size_green, header_size_blue;
int pa, pb, view_size;
char temp[4096];
int inptr;
char instring[1024];
int string_flag;
char input_string[1024];
int input_dest;
struct passwd *userinfo;
int init_flag = 0;
int v_text_pos;
int v_text_incr;
int image_bytes_per_color;
int zoom_x_size;
#ifdef MALLOC_TEST
struct mallinfo minfo; 
#endif // malloc test

/* defaults */

debug_flag = 0;

xpos = 0;
ypos = 0;

display_width = 800;
display_height = 600;

delta_x = 0;
delta_y = 0;
delta_z = 0;

xshift = 0;
yshift = 0;

//gamma_factor = 1.0 / 2.2;
gamma_factor = 1.0;

zoom_factor = 1;
enlarge_factor = 1;
help_flag = 0;

/* end defaults */

input_dest = DEST_NONE;

/* get user info */
userinfo = getpwuid(getuid() );

/* get home directory */ 
home_dir = strsave(userinfo -> pw_dir);
if(! home_dir)
	{
	fprintf(stdout,\
	"main(): could allocte space for home_dir\n");
	exit(1);
	}

user_name = strsave(userinfo -> pw_name);
if(! user_name)
	{
	fprintf(stdout,\
	"main(): could allocate space for user_name\n");
	exit(1);
	}

if(strcmp(user_name, "root") != 0)
	{
	fprintf(stderr,\
	"xmars3d: you need to be root to use this program, but you are %s aborting.\n", user_name);

	exit(1);
	}

/* update defaults */
load_general_settings("last_settings");

if(debug_flag)
	{
	fprintf(stderr,
	"main(): after load last_settings: filename_right=%s filename_left=%s\n",\
	filename_left, filename_right);
	}

#ifdef MALLOC_TEST
minfo = mallinfo();
fprintf(stderr,\
"minfo=%p  minfo.ordblks=%d minfo.fordblks=%d minfo.hblkhd=%d\n",\
minfo, minfo.ordblks, minfo.fordblks, minfo.hblkhd);

exit(1);
#endif // malloc test

while(1)
	{
	a = getopt(argc, argv, "e:g:h:l:r:vz:");
	if(a == -1) break;

	switch(a)
		{
		case 'e':
			enlarge_factor = atoi(optarg);
			if(enlarge_factor < 1)
				{
				fprintf(stderr, "xmars3d: enlarge factor should at least be 1, -e option, aborting.\n");	

				print_usage();
				exit(1);
				}
			break;
		case 'g':
			gamma_factor = atof(optarg);
			if(gamma_factor < 0.01)
				{
				fprintf(stderr, "xmars3d: gamma must be > .01, (-g option) aborting.\n");

				print_usage();
				exit(1);
				}
			break;
		case 'h':
			print_usage();
			exit(1);
			break;
		case 'l':
			filename_left = strsave(optarg);
			if(! filename_left)
				{
				fprintf(stderr,\
"xmars3d: no filename_left (-1 option) specified, or no ~/.xmars3d/setup/last_settings file, aborting.\n");

				print_usage();
				exit(1);
				}			
			break;
		case 'r':
			filename_right = strsave(optarg);
			if(! filename_right)
				{
				fprintf(stderr, "xmars3d: no filename_right (-2 option) specified, aborting.\n");

				print_usage();
				exit(1);
				}			
			break;
		case 'v':
			debug_flag = 1;
			break;
		case 'z':
			zoom_factor = atoi(optarg);
			if(zoom_factor < 1)
				{
				fprintf(stderr, "xmars3d: zoom factor should at least be 1, -z option, aborting.\n");	

				print_usage();
				exit(1);
				}
			break;
        case -1:
        	break;
		case '?':
			if(isprint(optopt) )
 				{
 				fprintf(stderr, "Unknown option `-%c'.\n", optopt);
 				}
			else
				{
				fprintf(stderr, "Unknown option character `\\x%x'.\n",\
				optopt);
				}
			print_usage();

			exit(1);
			break;			
		default:
			print_usage();

			exit(1);
			break;
		}/* end switch a */
	}/* end while getopt() */

if( (! filename_left) || (filename_left[0] == 0) )
	{
	fprintf(stderr,\
"xmars3d: no filename_left (-1 option) specified, or no ~/.xmars3d/setup/last_settings file, aborting.\n");
	print_usage();

	exit(1);
	}

/* open files */
/* open the pgm format file */
fptr1 = fopen(filename_left, "rb");
if(!fptr1)
	{
	fprintf(stdout,\
	"xmars3d: could not open left file %s for read\n", filename_left);

	strerror(errno);
	return 0;
	}

if(filename_right)
	{
	/* open the second pgm format file */
	fptr2 = fopen(filename_right, "rb");
	if(!fptr2)
		{
		fprintf(stdout,\
		"xmars3d: could not open right file %s for read\n", filename_right);

		strerror(errno);
		return 0;
		}
	}

/* allow IO for Asus LCD glasses */
iopl(3);

#ifdef DEFINED_SPEED
start_timeval = malloc(sizeof(struct timeval) );
if(! start_timeval)
	{
	fprintf(stderr, "could not allocate space for start_timeval\n");

	exit(1);
	}

current_timeval = malloc(sizeof(struct timeval) );
if(! current_timeval)
	{
	fprintf(stderr, "could not allocate space for current_timeval\n");

	exit(1);
	}
#endif

if( !ptc_open("Panteltje xmars3d", display_width, display_height) )
	{
	fprintf(stderr, "ptc_open() failed, aborting.\nAre you in at least 800x600 display mode?\n");

	exit(1);
	}

display_data1 = malloc(display_width * display_height * 4);
if(! display_data1)
	{
	fprintf(stderr, "malloc display_data1 failed, aborting.\n");

	exit(1);
	}

display_data2 = malloc(display_width * display_height * 4);
if(! display_data2)
	{
	fprintf(stderr, "malloc display_data2 failed, aborting.\n");

	exit(1);
	}

display_data3 = malloc(display_width * display_height * 4);
if(! display_data3)
	{
	fprintf(stderr, "malloc display_data3 failed, aborting.\n");

	exit(1);
	}

if(debug_flag)
	{
	fprintf(stderr, "display_data1=%p display_data2=%p\n", display_data1, display_data2);
	}

fill_gamma_table(gamma_factor);

/* no headers read yet */
header_size_left = 0;
header_size_right = 0;
header_size_red = 0;
header_size_green = 0;
header_size_blue = 0;

/* get initial view sizes */
view_width = display_width * zoom_factor;
view_height = display_height * zoom_factor;
view_size = view_height * view_width;

/* get left stereo picture size */
pgm_buffer_left =\
	pgm_and_ppm_to_char(fptr1, &x_size, &y_size, 0, 0, view_width, view_height,\
	&header_size_left, &input_image_bytes_per_color_left);

if(filename_right)
	{
	pgm_buffer_right =\
		pgm_and_ppm_to_char(fptr2, &x_size2, &y_size2, 0, 0, view_width, view_height,\
		&header_size_right, &input_image_bytes_per_color_right);
	}

if(debug_flag)
	{
	fprintf(stderr,\
	"pgm_buffer_left=%p pgm_buffer_right=%p x_size=%d y_size=%d x_size2=%d y_size2=%d\n",\
	pgm_buffer_left, pgm_buffer_right, x_size, y_size, x_size2, y_size2);
	}

/* zoom stuff */
ptri = pgm_buffer_left;
ptro = display_data1;
zoom_x_size = x_size;
image_bytes_per_color = input_image_bytes_per_color_left;
#include "zoom.c"
/* end zoom stuff */

if(filename_right)
	{
	/* zoom stuff */
	ptri = pgm_buffer_right;
	ptro = display_data2;
	zoom_x_size = x_size2;
	image_bytes_per_color = input_image_bytes_per_color_right;
	#include "zoom.c"
	/* end zoom stuff */

	} /* end if filename_right */


string_flag = 0;
inptr = 0; // -Wall
frame_count = 0;
update_params = 0; // -Wall
v_text_incr = 10;
while(1)
	{
	v_text_pos = 0;
//	fprintf(stderr, "key_press=%lu\n", key_press); 

#ifdef DEFINED_SPEED
	gettimeofday(start_timeval, NULL);
#endif
	if(string_flag && (key_press != 0) )
		{
		if( (key_press == 65293) || (inptr == 1022) ) // ENTER or string max size
			{
			string_flag = 0;
			instring[inptr] = 0;
						
			strcpy(input_string, instring);
			}
		else
			{
			if(key_press ==  65288) // backspace
				{
				if(inptr > 0) inptr--;
				instring[inptr] =  ' ';
				instring[inptr + 1] = 0;
				}
			else if( isalnum(key_press) || (key_press == '.') || (key_press == '/') )
				{
				instring[inptr] = key_press;
				inptr++;
				instring[inptr] = 0;
				}
			}
		}
	else
		{
		if(init_flag) update_params = 0;
		else update_params = 1;
		switch(key_press)
			{
			case ' ': // space bar
				delta_x = 0;
				delta_y = 0;
				update_params = 1;
				break;
			case 'E': // uppercase not returned by driver, neither shift release!
				break;
			case 'e':
				break;
			case 'f': // file menu positions
				#include "position_menu.c" // will set input_string
				input_dest = DEST_LOAD_POSITION;
				update_params = 1;
				break;
			case 'g': // gamma
				text_to_x11(\
       "gamma?", 10, 60, display_width, display_height, 0xff, 0xff, 0xff, 0x10 , 0x10, 0x10, display_data1);
				inptr = 0;
				instring[0] = 0;
				string_flag = 1;			
				input_string[0] = 0;
				input_dest = DEST_GAMMA;
				break;
			case 'h': // help
				help_flag = 1 - help_flag;
				update_params = 1;
				break;
			case 'l': // load position as
				text_to_x11(\
       "load position?", 10, 60, display_width, display_height, 0xff, 0xff, 0xff, 0x10 , 0x10, 0x10, display_data1);
				inptr = 0;
				instring[0] = 0;
				string_flag = 1;			
				input_string[0] = 0;
				input_dest = DEST_LOAD_POSITION;
				break;
			case 's': // save position as
				text_to_x11(\
       "save position as?", 10, 60, display_width, display_height, 0xff, 0xff, 0xff, 0x10 , 0x10, 0x10, display_data1);
				inptr = 0;
				instring[0] = 0;
				string_flag = 1;			
				input_string[0] = 0;
				input_dest = DEST_SAVE_POSITION;
				break;
			case 'v': // save screen as ppm in color
				delta_x = 0;
				delta_y = 0;
				delta_z = 0;
				/* make a copy without following dialog text */
				memcpy(display_data3, display_data1, 4 * display_width * display_height);
				text_to_x11(\
       "save screen as?", 10, 60, display_width, display_height, 0xff, 0xff, 0xff, 0x10 , 0x10, 0x10, display_data1);
				inptr = 0;
				instring[0] = 0;
				string_flag = 1;			
				input_string[0] = 0;
				input_dest = DEST_SAVE_SCREEN_COLOR;
				break;
			case 'w': // save screen as pgm in BW
				delta_x = 0;
				delta_y = 0;
				delta_z = 0;
				/* make a copy without following dialog text */
				memcpy(display_data3, display_data1, 4 * display_width * display_height);
				text_to_x11(\
       "save screen as?", 10, 60, display_width, display_height, 0xff, 0xff, 0xff, 0x10 , 0x10, 0x10, display_data1);
				inptr = 0;
				instring[0] = 0;
				string_flag = 1;			
				input_string[0] = 0;
				input_dest = DEST_SAVE_SCREEN_BW;
				break;
			case 'x': // x position
				text_to_x11(\
       "x position?", 10, 60, display_width, display_height, 0xff, 0xff, 0xff, 0x10 , 0x10, 0x10, display_data1);
			
				delta_x = 0;
				inptr = 0;
				instring[0] = 0;
				string_flag = 1;			
				input_string[0] = 0;
				input_dest = DEST_X;
				break;
			case 'y': // y position
				text_to_x11(\
       "y position?", 10, 60, display_width, display_height, 0xff, 0xff, 0xff, 0x10 , 0x10, 0x10, display_data1);
			
				delta_x = 0;
				inptr = 0;
				instring[0] = 0;
				string_flag = 1;			
				input_string[0] = 0;
				input_dest = DEST_Y;
				break;
			case 'z': // z position (zoom factor = altitude)
				text_to_x11(\
       "z position?", 10, 60, display_width, display_height, 0xff, 0xff, 0xff, 0x10 , 0x10, 0x10, display_data1);
			
				inptr = 0;
				instring[0] = 0;
				string_flag = 1;			
				input_string[0] = 0;
				input_dest = DEST_Z;
				break;
			case '[':// gamma down
				if(gamma_factor > 0.01 )
					{
					gamma_factor -= 0.01;
					fill_gamma_table(gamma_factor);
					update_params = 1;
					}
				break;
			case ']': // gamma up
				if(gamma_factor < 4.0 )
					{
					gamma_factor += 0.01;
					fill_gamma_table(gamma_factor);
					update_params = 1;
					}
				break;
			case 65360: // grey home
				xshift = 0;
				yshift = 0;
				update_params = 1;
				break;
			case 65361: // grey cu left
				if(delta_x > -255)
					{
					delta_x--;
					update_params = 1;
					}
				break;
			case 65362: // grey cu up
				if(delta_y > -255)
					{
					delta_y--;
					update_params = 1;
					}
				break;
			case 65363: // grey cu right
				if(delta_x < 255)
					{
					delta_x++;
					update_params = 1;
					}
				break;
			case 65364: // grey cu down
				if(delta_y < 255)
					{
					delta_y++;
					update_params = 1;
					}
				break;
			case 65365: // grey page up
				if(enlarge_factor == 1)
					{
					if(zoom_factor < 64)
						{
						zoom_factor *= 2;

						xpos -= (view_width / 2);
						ypos -= (view_height / 2);
	
						update_params = 1;
						}
					}
				break;
			case 65366: // grey page down
				if(enlarge_factor == 1)
					{
					if(zoom_factor > 1)
						{
						zoom_factor /= 2;

						xpos += (view_width / 4);
						ypos += (view_height / 4);

						update_params = 1;
						}
					}
				break;	
			case 65430: // cu left
				update_params = 1;
				xshift--;
				break;
			case 65431: // cu up
				update_params = 1;
				yshift--;	
				break;
			case 65432:	// cu right
				update_params = 1;
				xshift++;
				break;
			case 65433: // cu down
				yshift++;
				update_params = 1;
				break;
			case 65470: // F1 decrease maginfication by 2
				if(zoom_factor == 1)
					{
					if(enlarge_factor > 1)
						{
						xpos -= (display_width / (enlarge_factor * 2) );
						ypos -= (display_height / (enlarge_factor * 2) );
						enlarge_factor /= 2; 
						update_params = 1;
						}
					}	
			break;
			case 65471: // F2 increase magnification by 2
				if(zoom_factor == 1)
					{
					if(enlarge_factor < 64)
						{
						enlarge_factor *= 2;
						xpos += (display_width / (enlarge_factor * 2) );
						ypos += (display_height / (enlarge_factor * 2) );
						update_params = 1;
						}
					}
				break;
			case 65472: // F3
				break;
			case 65473: // F4 
				break;
			case 65505: // shift key
//fprintf(stderr, "SHIFT (key_press & 0xff00) >> 8 = %d\n", (key_press & 0xff00) >> 8);
				break;
			default:
				break;
			} /* end switch key_press */
		}
	key_press = 0;

	if(zoom_factor > 1) enlarge_factor = 1;

	if(string_flag)
		{
		text_to_x11(\
		instring, 160, 60, display_width, display_height, 0xff, 0xff, 0xff, 0x10 , 0x10, 0x10, display_data1);	
		}

	if(input_string[0] != 0)
		{
		switch(input_dest)
			{
			case DEST_GAMMA:
				gamma_factor = atof(input_string);
				fill_gamma_table(gamma_factor);
				break;
			case DEST_SAVE_SCREEN_COLOR:
//				int x11_to_ppm(int *buffer, int xpos, int ypos, int width, int height, char *filename)
				/* use the screen copy without dialog text */
				x11_to_ppm(display_data3, xpos, ypos, display_width, display_height, input_string);
				break;
			case DEST_SAVE_SCREEN_BW:
//				int x11_to_pgm(int *buffer, int xpos, int ypos, int width, int height, char *filename)
				/* use the screen copy without dialog text */
				x11_to_pgm(display_data3, xpos, ypos, display_width, display_height, input_string);
				break;
			case DEST_SAVE_POSITION:
				save_general_settings(input_string);
				break;
			case DEST_LOAD_POSITION:
				/* need to r-intitialize everything as picture(s) may have changed */
				#include "reload.c"
				break;
			case DEST_X:
				xpos = atoi(input_string);			
				break;
			case DEST_Y:
				ypos = atoi(input_string);
				break;
			case DEST_Z:
				zoom_factor = atoi(input_string);
				if(zoom_factor < 1) zoom_factor = 1;
				break;
			default:
				break;
			} /* end switch input_dest */

		input_string[0] = 0;
		input_dest = DEST_NONE;
		
		update_params = 1;
		} /* end if some input */

	xpos += delta_x;
	ypos += delta_y;

	if(xpos >= x_size - view_width)
		{
		xpos = x_size - view_width;
		delta_x = 0; // stop
		}
	if(xpos < 0)
		{
		xpos = 0;
		delta_x = 0; // stop
		}

	if(ypos >= (y_size - view_height) )
		{
		ypos = y_size - view_height;
		delta_y = 0; // stop
		}

	if(ypos < 0)
		{
		ypos = 0;
		delta_y = 0; // stop
		}

	xpos2 = xpos + xshift;
	ypos2 = ypos + yshift;
	
	if(xpos2 > x_size2 - view_width) xpos2 = x_size2 - view_width;	
	if(xpos2 < 0) xpos2 = 0;

	if(ypos2 > (y_size2 - view_height) ) ypos2 = y_size2 - view_height;
	if(ypos2 < 0) ypos2 = 0;

	view_width = display_width * zoom_factor;
	view_height = display_height * zoom_factor;
	view_size = view_height * view_width;

	/* update display data when we are moving */
	if(delta_x || delta_y || update_params) 
		{
		if(debug_flag)
			{
			fprintf(stderr,\
			"main(): xpos=%d ypos=%d width=%d heigh=%d delta_x=%d delta_y=%d zoom_factor=%d\n",\
			xpos, ypos, view_width, view_height, delta_x, delta_y, zoom_factor);
			}

		free(pgm_buffer_left);

		pgm_buffer_left =\
		pgm_and_ppm_to_char(fptr1, &x_size, &y_size, xpos, ypos, view_width, view_height,
		&header_size_left, &input_image_bytes_per_color_left);

		/* zoom stuff */
		ptri = pgm_buffer_left;
		ptro = display_data1;
		zoom_x_size = x_size;
		image_bytes_per_color = input_image_bytes_per_color_left;
		#include "zoom.c"
		/* end zoom stuff */

		if(filename_right)
			{
			free(pgm_buffer_right);

			pgm_buffer_right =\
			pgm_and_ppm_to_char(fptr2, &x_size2, &y_size2, xpos2, ypos2, view_width, view_height,\
			&header_size_right, &input_image_bytes_per_color_right);

			/* zoom stuff */
			ptri = pgm_buffer_right;
			ptro = display_data2;
			zoom_x_size = x_size2;
			image_bytes_per_color = input_image_bytes_per_color_right;
			#include "zoom.c"
			/* end zoom stuff */

			} /* end if filename_right */ 

		} /* end if update_params or delta_x or delta_y */

	sprintf(temp,\
	"xmars3d-%s filename_left=%s filename_right=%s",\
	VERSION, filename_left, filename_right);		

	if( ! filename_right)
		{
		sprintf(temp,\
		"xmars3d-%s filename=%s",\
		VERSION, filename_left);

		text_to_x11(\
	temp, 0, v_text_pos, display_width, display_height, 0xff, 0xff, 0xff, 0x10 , 0x10, 0x10, display_data1);
		v_text_pos += v_text_incr;
		}
	else
		{
		sprintf(temp,\
		"xmars3d-%s filename_left=%s",\
		VERSION, filename_left);

		text_to_x11(\
		temp, 0, 0, display_width, display_height, 0xff, 0xff, 0xff, 0x10 , 0x10, 0x10, display_data1);
		v_text_pos += v_text_incr;


		sprintf(temp,\
		"filename_right=%s",\
		filename_right);

		text_to_x11(\
		temp, 0, v_text_pos, display_width, display_height, 0xff, 0xff, 0xff, 0x10 , 0x10, 0x10, display_data1);
		v_text_pos += v_text_incr;
		}


	sprintf(temp,\
	"xpos=%d ypos=%d height=%d enlarge=%d xshift=%d yshift=%d gamma=%.2f delta_x=%d delta_y=%d h=help",\
	xpos, ypos, zoom_factor, enlarge_factor, xshift, yshift, gamma_factor, delta_x, delta_y);		

	text_to_x11(\
		temp, 0, v_text_pos, display_width, display_height, 0xff, 0xff, 0xff, 0x10 , 0x10, 0x10, display_data1);
		v_text_pos += v_text_incr;
		
	if(help_flag)
		{
		sprintf(temp,\
		"grey cu = thrusters, white cu = shift(stereo), '[' and ']' = gamma, grey pu / pdown = height");

		text_to_x11(\
	temp, 10, v_text_pos, display_width, display_height, 0xff, 0xff, 0xff, 0x10 , 0x10, 0x10, display_data1);
		v_text_pos += v_text_incr;


		sprintf(temp,\
		"space bar = stop  g = gamma  l = load  s = save  x = set xpos  y = set ypos  z = set zoom");

		text_to_x11(\
	temp, 10, v_text_pos, display_width, display_height, 0xff, 0xff, 0xff, 0x10 , 0x10, 0x10, display_data1);
		v_text_pos += v_text_incr;


		sprintf(temp,\
		"if zoom = 1, then F2 enlarges and F1 reduces to 1:1 pixels.");

		text_to_x11(\
	temp, 10, v_text_pos, display_width, display_height, 0xff, 0xff, 0xff, 0x10 , 0x10, 0x10, display_data1);
		v_text_pos += v_text_incr;


		sprintf(temp,\
		"v = save screen in color as .ppm  w = save screen in BW as .pgm  f = list choices  ESC exits.");

		text_to_x11(\
	temp, 10, v_text_pos, display_width, display_height, 0xff, 0xff, 0xff, 0x10 , 0x10, 0x10, display_data1);
		v_text_pos += v_text_incr;
		}

	/* this later so we write in display after it is build */
	if(update_params)
		{
		save_general_settings("last_settings");
		}

	ptc_update(display_data1);
	set_lcd(0);

	init_flag = 1;

#ifdef DEFINED_SPEED
	while(1)
		{
		/* get elapsed time */
		gettimeofday(current_timeval, NULL);

		lcurrent_usecs = current_timeval -> tv_usec + (1000000 * current_timeval -> tv_sec);

		lstart_usecs = start_timeval -> tv_usec + (1000000 * start_timeval -> tv_sec);

		ldiff_usecs = lcurrent_usecs - lstart_usecs;

		if(ldiff_usecs > 20000) break;
//		fprintf(stderr, "EARLY1 %lu\n", ldiff_usecs);
		}
#endif

	if(! filename_right) continue;

	ptc_update(display_data2);
	set_lcd(1);

#ifdef DEFINED_SPEED
	while(1)
		{
		/* get elapsed time */
		gettimeofday(current_timeval, NULL);

		lcurrent_usecs = current_timeval -> tv_usec + (1000000 * current_timeval -> tv_sec);

		lstart_usecs = start_timeval -> tv_usec + (1000000 * start_timeval -> tv_sec);

		ldiff_usecs = lcurrent_usecs - lstart_usecs;

		if(ldiff_usecs > 40000) break;

//		fprintf(stderr, "EARLY2 %lu\n", ldiff_usecs);
		}
#endif

	frame_count++;	

//	if(frame_count > (25 * 60) ) break;

	} /* end while frames */

ptc_close();

exit(0);
} /* end function main */


